/**
 * Classe héritée de Courrier gérant l'affranchissement d'un colis.
 */
public class Colis extends Courrier {

    /**
     * Variables de classe : les différents coefficients associés au type Colis. 
     */
    static final double COEF_POIDS = 1.0;
    static final double COEF_VOLUME = 1.5;
    static final double VOLUME_MAX = 50;
    static final int POIDS_MAX = 30000; // 30 kg
    /**
     * Variables d'instance : le volume du colis.
     */
    private double volume;
    /**
     * Constructeur de l'objet Colis.
     * @param poids Poids du colis en grammes
     * @param express Le mode d'envoi (normal : false ou express : true)
     * @param adresseDestination L'adresse de destination du colis. 
     * @param volume Le volume en litres. 
     */
    public Colis(int poids, boolean express, String adresseDestination, double volume) {
        super(poids, express, adresseDestination);
        this.volume = volume;
    }

    /**
     * Renvoie vrai si et seulement si le colis est valide, c'est à dire si son volume n'excède pas le volume maximum.
     */
    @Override
    public boolean estValide() {
        return super.estValide() && (this.volume <= VOLUME_MAX);
    }
    /**
     * Ajoute le volume à la méthode toString de la classe abstraite courrier. 
     */
    @Override
    public String toString() {
        String res = super.toString() +
                "Volume : " + this.volume + " litres";
        if (this.volume > VOLUME_MAX) {
            res += "   VOLUME TROP IMPORTANT";
        }
        return res + "\n";
    }
    @Override
    public int poidsMax() {
        return Colis.POIDS_MAX;
    }
    @Override
    public double tarifNormal() {
        return 4.0 + COEF_VOLUME * this.volume + COEF_POIDS * (this.poids / 1000.0);  // 1000.0 car sinon division entière
    }
}
