
/**
 * Définition du type abstrait Courrier
 */
public abstract class Courrier {

    /**
     * Coéfficient multiplicateur pour lettre express
     */
    public static final double COEFF_EXPRESS = 2.0;
    /**
     * Le poids du courrier
     */
    protected int poids;
    /**
     * Le mode d'expédition true : express, false normal
     */
    protected boolean express;
    /**
     * L'adresse de destination
     */
    protected String adresseDestination;

    /**
     * Construit un objet abstrait courrier
     * 
     * @param poids               Le poids de courrier (en g)
     * @param mode                Son mode d'expédition (true : express, false : normal)
     * @param adresse_destination Son adresse de destination.
     */
    public Courrier(int poids, boolean express, String adresse_destination) {
        this.poids = poids;
        this.express = express;
        this.adresseDestination = adresse_destination;
    }

    /**
     * 
     * @return L'adresse de destination du courrier.
     */
    public String getAdresseDestination() {
        return this.adresseDestination;
    }

    /**
     * 
     * @return "" si est le courrier est valide (l'adresse de  destination est non vide 
     *    et le poids du courrier est inférieur au poids max autorisé)
     */
    public boolean estValide() {
        return  (this.adresseDestination != null && ! this.adresseDestination.isBlank() && this.poids <= this.poidsMax()) ;
    }

    /**
     * 
     * @return le poids max autorisé pour ce courrier
     */
    public abstract int poidsMax() ;

    /**
     * 
     * @return La partie invariable de l'affranchissement d'un courrier.
     */
    public double valeurAffranchissement() {
        if (this.estValide()) {
            if (this.express) {
                return COEFF_EXPRESS * this.tarifNormal();
            }
            return this.tarifNormal();
        }
        return 0.0;
    }

    @Override
    /**
     * Renvoie une chaîne de caractère comportant le nom et la valeur de tous les
     * attributs globaux du courrier.
     */
    public String toString() {
        String res =  this.getClass().getName() + "\n";
        if (this.estValide() ) {
            res += "Prix : " + this.valeurAffranchissement() + " €\n";
        }
        res += "Poids : " + this.poids + " g";
        if (poids > this.poidsMax() ) {
            res += "  POIDS TROP ELEVE";
        }
        res += "\nExpress : " + ((this.express)?"oui":"non") + "\n";
        res += "Destination : " + this.adresseDestination ;
        if (adresseDestination == null || adresseDestination.isBlank()) {
            res += "  ADRESSE MANQUANTE";
        } 
        return res + "\n";
    }

    /**
     * Méthode abstraite laissée aux classes filles pour calculer le prix
     * spécifique du courrier au tarif Normal
     * 
     * @return Prix spécifique du courrier au tarif Normal
     */
    public abstract double tarifNormal();
}
