/**
 * Classe modélisant la machine à affranchir le courrier.
 */
public class Machine {  
    /**
     *  le nombre de courriers que contient la machine
     */
    private int nbreCourriers;

    /**
     *  le nombre de courriers invalide qui ont été refusés
     */
    private int nbreCourriersInvalides;

    /**
     * liste des courriers à affranchir.
     */
    private Courrier[] courriers;

    /**
     * Constructeur de l'objet Machine
     * @param capacite Capacité de la machine.
     */
    public Machine(int capacite) {
        this.courriers = new Courrier[capacite];
    }
    /**
     * Permet d'ajouter un courrier à la machine
     * @param c Le courrier à ajouter à la machine
     */
    public void deposerCourrier(Courrier c) {

        if (this.nbreCourriers >= courriers.length) {
            throw new IllegalArgumentException("Capacité dépassée");
        }
        System.out.print(c);
        if (c.estValide()) {
            this.courriers[nbreCourriers++] = c;
            System.out.println("COURRIER ACCEPTE\n");
        } else {
            System.out.println("COURRIER REFUSE\n");
            nbreCourriersInvalides++;
        }
    }

    /**
     * Calcule le montant total de l'affranchissement
     * @return montant total d'affranchissement
     */
    public double valeurCourrier() {
        double montant = 0;
        for (int i = 0 ; i < nbreCourriers; i++) {
            montant += courriers[i].valeurAffranchissement();
        }
        return montant;
    }

    /**
     * Affiche l'ensemble des courriers de la machine. 
     */
    public void detailsCourriers() {
        System.out.println("-----------------------------------------------------");
        System.out.println("Nbre total de courriers :  " + getNbreCourriers()) ;
        System.out.printf("Montant total d'affranchissement : %.2f €\n" ,valeurCourrier()) ;
        System.out.println("Nbre total refusés : " +  getNbreCourriersInvalides()) ;
        System.out.println("-----------------------------------------------------");
        System.out.println("details des courriers\n");
        for (int i = 0 ; i < nbreCourriers; i++) {
            System.out.println(courriers[i]);
        }
    }

    public int getNbreCourriers() {
        return nbreCourriers;
    }

    public int getNbreCourriersInvalides() {
        return nbreCourriersInvalides;
    }

}
